var express = require('express');
var app = express();

// pour les formulaires multiparts
var multer = require('multer');
var multerData = multer();

// Mon module de connexion à MongoDB
const mongoDBModule = require('./app_modules/crud-mongo');

// Autoriser le CROSS ORIGIN
app.use(function (req, res, next) {
    res.header("Access-Control-Allow-Origin", "*");
    res.header("Access-Control-Allow-Headers", "Origin, X-Requested-With, Content-Type, Accept");
    res.header("Access-Control-Allow-Methods", "POST, GET, OPTIONS, PUT, DELETE");

    next();
});

// avec cette ligne, index.html est par defaut la page d’accueil
app.use(express.static('public')); // ICI IMPORTANT !


// Avec la règle ci-dessous on peut redéfinir la page d’accueil
app.get('/', function (req, res) { // Page d’accueil
    res.sendFile(__dirname + "/public/index.html");
});

let restaurants = [{
        nom: "Mac Donald",
        cuisine: "fast food",
        id: 1
    },
    {
        nom: "Restau Tacos",
        cuisine: "Lyonnaise",
        id: 2
    },
    {
        nom: "Mac Donald 2",
        cuisine: "fast food 2",
        id: 3
    },
    {
        nom: "Restau Tacos 2",
        cuisine: "Lyonnaise 2",
        id: 4
    },
    {
        nom: "Mac Donald 3",
        cuisine: "fast food 3",
        id: 5
    },
    {
        nom: "Restau Tacos 3",
        cuisine: "Lyonnaise 3",
        id: 6
    },
    {
        nom: "Mac Donald 4",
        cuisine: "fast food 4",
        id: 7
    },
    {
        nom: "Restau Tacos 4",
        cuisine: "Lyonnaise 4",
        id: 8
    },
    {
        nom: "Mac Donald 5",
        cuisine: "fast food 5",
        id: 9
    },
    {
        nom: "Restau Tacos 5",
        cuisine: "Lyonnaise 5",
        id: 10
    }
]

app.get('/api/restaurantsold', (req, res) => {
    // paramètres optionnels passé en tant que 
    // query strings: ex /api/restaurants?page=2@pagesize=5&nom=Pizza

    let page = req.query.page || 0;
    let pagesize = req.query.pagesize || 10;
    let nom = req.query.nom || "";
    let cuisine = req.query.cuisine || "";

    let reponse = {
        msg: "get restaurants page=" +
            page + " pagesize = " +
            pagesize + " nom = " + nom + " cuisine = " +
            cuisine,
        data: restaurants,
        err: ""
    }
    res.send(JSON.stringify(reponse));
})

app.get('/api/restaurants', (req, res) => {
    // paramètres optionnels passé en tant que 
    // query strings: ex /api/restaurants?page=2@pagesize=5&nom=Pizza

    let page = req.query.page || 0;
    let pagesize = req.query.pagesize || 10;
    let nom = req.query.nom || "";
    let cuisine = req.query.cuisine || "";

    mongoDBModule.findRestaurants(page, pagesize, nom, cuisine, (data, count) => {
        var reponse = {
            msg: "restaurant recherchés avec succès",
            data: data,
            count:count
        }
        res.send(JSON.stringify(reponse));
    });
})

app.get('/api/restaurants/:id', (req, res) => {
    // L'id est dans l'URI /api/restaurants/2 par ex:
    let id = req.params.id;

    let restaurantCherche = null;

    for (let i = 0; i < restaurants.length; i++) {
        if (restaurants[i].id == id) {
            restaurantCherche = restaurants[i];
            break;
        }
    }

    let reponse = {
        msg: "Je cherche le restaurant id = " + id,
        data: restaurantCherche,
        err: ""
    }
    res.send(JSON.stringify(reponse));
})
app.post('/api/restaurants', multerData.fields([]), function (req, res) {
    // On supposera qu'on ajoutera un restaurant en 
    // donnant son nom et sa cuisine. On va donc 
    // recuperer les données du formulaire d'envoi
    // les params sont dans req.body même si le formulaire
    // est envoyé en multipart

    let nom = req.body.nom;
    let cuisine = req.body.cuisine;
    let id = Math.random();
    restaurants.push({
        nom: nom,
        cuisine: cuisine,
        id: id
    })

    let reponse = {
        msg: "J'insère un restaurant nom = " + nom + " cuisine = " + cuisine,
        data: "id du restaurant inséré " + id,
        err: ""
    }

    res.send(JSON.stringify(reponse));
});

app.put('/api/restaurants/:id', multerData.fields([]), (req, res) => {
    // MODIFICATION

    let nom = req.body.nom;
    let cuisine = req.body.cuisine;
    let id = req.params.id;

    restaurants[id - 1].nom = nom;
    restaurants[id - 1].cuisine = cuisine;

    let reponse = {
        msg: "Je modifie le restaurant d'id = " + id +
            " avec nom = " + nom + " cuisine = " + cuisine,
        data: restaurants[id - 1],
        err: ""
    }

    res.send(JSON.stringify(reponse));
});


app.delete('/api/restaurants/:id', (req, res) => {
    // SUPPRESSION

    let id = req.params.id;

    restaurants.splice(id - 1, 1); // pos, nb elements à suprimmer

    let reponse = {
        msg: "J'ai supprimé le restaurant d'id = " + id,
        data: 1, // nb de suppression
        err: ""
    }

    res.send(JSON.stringify(reponse));
});

var server = app.listen(8085, function () {
    var host = server.address().address;
    var port = server.address().port;
    console.log("Serveur écoute sur 8085")
})